import type {TurboModule} from 'react-native';
import {TurboModuleRegistry} from 'react-native';
import type { UnsafeObject } from 'react-native/Libraries/Types/CodegenTypes';

// Add all the methods marked with @ReactMethod in BugseeNativeBridge.java
export interface Spec extends TurboModule {
   
    setFlag(flagName: string, flagState: boolean): void;
    launch(appToken: string, options: UnsafeObject): Promise<void>;
    stop(): Promise<void>;
    relaunch(options: UnsafeObject): Promise<void>;
    log(params?: { text: string, level?: number }): void;
    event(params?: { name: string, params?: UnsafeObject }): void;
    trace(params?: { name: string, value: unknown }): void;
    pause(): void;
    resume(): void;
    toggleProtected(viewId: number, isProtected: boolean): void;
    showReportDialog(params?: { summary?: string, description?: string, severity?: number }): void;
    upload(params?: { summary: string, description?: string, severity?: number, labels?: string[], includeVideo?: boolean }): void;
    setEmail(params?: { email?: string | null | undefined }): void;
    clearEmail(): void;
    getEmail(): Promise<string>;
    setAttribute(params?: { name: string, value: unknown }): void;
    getAttribute(name: string): Promise<unknown>;
    clearAttribute(params?: { name: string }): void;
    clearAllAttributes(): void;
    showFeedbackUI(): void;
    setDefaultFeedbackGreeting(message: string): void;
    logException(exception: string, isHandled?: boolean, options?: UnsafeObject | null | undefined): Promise<void>;
    deleteCollectedDataOnDevice(): Promise<boolean>;
    addSecureRect(left: number, top: number, width: number, height: number): void;
    removeSecureRect(left: number, top: number, width: number, height: number): void;
    getAllSecureRects(): Promise<number[][]>;
    removeAllSecureRects(): void;
    getVersions(): Promise<UnsafeObject>;
    getLaunchOptions(): Promise<UnsafeObject>;
    hideKeyboard(isHidden: boolean): void;
    logNetworkEvent(networkJson: string): void;
    setAttachments(attachmentsJson: string): void;
    getAppearanceProperty(propertyName: string): Promise<string>;
    setAppearanceProperty(appearanceData: UnsafeObject): Promise<void>;
    testCrash(): void;
    captureViewHierarchy(): void;
    addListener(eventName: string): void;
    removeListeners(count: number): void;
}


export default TurboModuleRegistry.getEnforcing<Spec>(
    'Bugsee',
);